==> sptk-3.5.7.14.tbz2: sptk-3.5.7.14/src/utils/CThread.cpp <==
/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CThread.cpp  -  description
                             -------------------
    begin                : Thu Jul 12 2001
    copyright            : (C) 2001-2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CThread.h>
#include <sptk3/CException.h>

#include <stdio.h>
#include <string.h>
#include <iostream>

using namespace std;
using namespace sptk;

extern "C" {
  typedef void* (*CThreadStartFunction)(void*);
}

#ifndef _WIN32
void *CThread::threadStart(void *p) {
   pthread_cleanup_push(threadCleanup,p);
   CThread *thread = (CThread *)p;
   thread->runThread();
   pthread_cleanup_pop(1);
   return NULL;
}
#else
unsigned __stdcall CThread::threadStart(void *p) {
   CThread *thread = (CThread *)p;
   thread->runThread();
   return 0;
}
#endif

///@todo implement windows version of cleanup registration
void CThread::threadCleanup(void *p) {
   CThread *thread = (CThread *)p;
   thread->onThreadExit();
}

CThread::CThread(string name,bool politeMode,CFileLog* fileLog) {
   m_name = name;
   m_terminated = false;
   m_running = false;
   m_politeMode = politeMode;
   m_log = fileLog;
   if (m_log) *m_log << "Thread " << m_name << ": constructor completed" << endl;
}

CThread::~CThread() {
   terminate();
   if (m_log) *m_log << "Thread " << m_name << ": destructor completed" << endl;
}

void CThread::terminate()  {
   if (m_terminated) return;
   m_terminated = true;

   if (m_log) *m_log << "Thread " << m_name << ": termination started" << endl;
   if (m_running) {
      if (m_politeMode) {
#ifndef _WIN32
         // wait till the thread is stopping
         pthread_join(m_thread,0);
#else
         WaitForSingleObject(m_thread,INFINITE);
         m_thread = 0;
#endif
      } else {
#ifndef _WIN32
         // stop this thread _now_
         pthread_cancel(m_thread);
#else
         CloseHandle(m_thread);
#endif
      }
   }
   if (m_log) *m_log << "Thread " << m_name << ": termination completed" << endl;
}

void CThread::createThread() {
#ifndef _WIN32
   int rc = pthread_create(&m_thread, NULL, (CThreadStartFunction) CThread::threadStart, (void *) this);
   if (rc)
      throw CException("Can't create a new thread");
#else
   unsigned threadID;
   m_thread = (HANDLE) _beginthreadex(0L, 2048, CThread::threadStart, (void *) this, 0, &threadID);
   if (!m_thread)
      throw CException("Can't create a new thread");
#endif
   if (m_log) *m_log << "Thread " << m_name << ": started" << endl;
}

void CThread::destroyThread() {
   if (m_log) *m_log << "Thread " << m_name << ": destroy started" << endl;
#ifndef _WIN32
   pthread_exit(NULL);
#else
   _endthreadex(0);
#endif
   if (m_log) *m_log << "Thread " << m_name << ": destroy completed" << endl;
}

void CThread::runThread() {
   m_running = true;
   threadFunction();
   m_running = false;
   destroyThread();
}

void CThread::run() {
   m_terminated = false;
   m_running = true;
   createThread();
}

void CThread::msleep(int msec) {
#ifndef _WIN32
   usleep(msec * 1000L);
#else
   Sleep(msec);
#endif
}

==> sptk-3.5.8.8.tbz2: sptk-3.5.8.8/src/utils/CThread.cpp <==
/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CThread.cpp  -  description
                             -------------------
    begin                : Thu Jul 12 2001
    copyright            : (C) 2001-2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CThread.h>
#include <sptk3/CException.h>

#include <stdio.h>
#include <string.h>
#include <iostream>
#include <stdint.h>

#ifndef _WIN32
#include <limits.h>
#endif

using namespace std;
using namespace sptk;

extern "C" {
  typedef void* (*CThreadStartFunction)(void*);
}

#ifndef _WIN32
void *CThread::threadStart(void *p) {
   pthread_cleanup_push(threadCleanup,p);
   CThread *thread = (CThread *)p;
   thread->runThread();
   pthread_cleanup_pop(1);
   return NULL;
}
#else
unsigned __stdcall CThread::threadStart(void *p) {
   CThread *thread = (CThread *)p;
   thread->runThread();
   return 0;
}
#endif

///@todo implement windows version of cleanup registration
void CThread::threadCleanup(void *p) {
   CThread *thread = (CThread *)p;
   thread->onThreadExit();
}

CThread::CThread(string name,bool politeMode,CFileLog* aLog,uint32_t stackSize) {
   m_name = name;
   m_terminated = false;
   m_running = false;
   m_politeMode = politeMode;
   m_log = aLog;
   m_stackSize = stackSize;
   if (m_log) *m_log << "Thread " << m_name << ": constructor completed" << endl;
}

CThread::~CThread() {
   terminate();
   if (m_log) *m_log << "Thread " << m_name << ": destructor completed" << endl;
}

void CThread::terminate()  {
   if (m_terminated) return;
   m_terminated = true;

   if (m_log) *m_log << "Thread " << m_name << ": termination started" << endl;
   if (m_running) {
      if (m_politeMode) {
#ifndef _WIN32
         // wait till the thread is stopping
         pthread_join(m_thread,0);
#else
         WaitForSingleObject(m_thread,INFINITE);
         m_thread = 0;
#endif
      } else {
#ifndef _WIN32
         // stop this thread _now_
         pthread_cancel(m_thread);
#else
         CloseHandle(m_thread);
#endif
      }
   }
   if (m_log) *m_log << "Thread " << m_name << ": termination completed" << endl;
}

void CThread::createThread() {
#ifndef _WIN32
    pthread_attr_t tattr;

    /* initialized with default attributes */
    pthread_attr_init(&tattr);
    /* setting the size of the stack also */
    if (m_stackSize) {
        int stackSize = m_stackSize > PTHREAD_STACK_MIN ? m_stackSize : PTHREAD_STACK_MIN;
        pthread_attr_setstacksize(&tattr, stackSize);
    }
    int rc = pthread_create(&m_thread, &tattr, (CThreadStartFunction) CThread::threadStart, (void *) this);
    if (rc)
        throw CException("Can't create a new thread");
#else
    unsigned threadID;
    m_thread = (HANDLE) _beginthreadex(0L, m_stackSize, CThread::threadStart, (void *) this, 0, &threadID);
    if (!m_thread)
        throw CException("Can't create a new thread");
#endif
    if (m_log) *m_log << "Thread " << m_name << ": started" << endl;
}

void CThread::destroyThread() {
   if (m_log) *m_log << "Thread " << m_name << ": destroy started" << endl;
#ifndef _WIN32
   //pthread_exit(NULL);
#else
   _endthreadex(0);
#endif
   if (m_log) *m_log << "Thread " << m_name << ": destroy completed" << endl;
}

void CThread::runThread() {
   m_running = true;
   threadFunction();
   m_running = false;
   destroyThread();
}

void CThread::run() {
   m_terminated = false;
   m_running = true;
   createThread();
}

void CThread::msleep(int msec) {
#ifndef _WIN32
   usleep(msec * 1000L);
#else
   Sleep(msec);
#endif
}
